<?php
if(!class_exists("Gc_Message_Box_Admin_Controller")){
	class Gc_Message_Box_Admin_Controller implements Gc_MessageBox_Controller_Interface {
        protected $namespace;
        protected $options;
		protected $this_plugin = 'gc-message-box/main.php';

        public function __construct($namespace){
            $this->namespace = $namespace;

        }
        public function initialize(){
            $this->options = Gc_MessageBox_CF::create("Option_Repository_Factory")->get_instance()->get_namespace($this->namespace);
        }
        public function initialize_hooks(){
            if(Gc_MessageBox_Util::is_plugin_page()){
                add_action( 'init', array($this, 'scripts_init') );
                $this->initialize_less_js();
            }
            add_action( 'wp_ajax_gc-message-box-group', array($this,'group') );
		    add_action( 'init', array($this, 'adminbar_init') );
		    add_action( 'admin_init', array($this, 'admin_init') );
		    add_action( 'admin_menu', array($this, 'add_sub_menu') );
			add_action( 'wp_ajax_nopriv_gc-message-box', array($this,'info') );

            if($this->options->get("enable_remote_configuration")->get_value() == 1)  {
                add_action( 'wp_ajax_nopriv_gc-message-box-remote', array($this,'remote') );
                add_action( 'wp_ajax_gc-message-box-remote', array($this,'remote') );
            }

            $this->initialize_getconversion_remote();            

            add_filter( 'plugin_action_links', array($this, 'add_action_link'), 10, 2 );
            add_action( 'wp_before_admin_bar_render', array($this, 'init_admin_bar') );

		}

        public function adminbar_init(){
            Gc_Message_Box_Admin_Bar::script_init();
        }

        protected function initialize_getconversion_remote(){
            add_action( 'wp_ajax_gc-message-box-activate', array($this,'activate') );
            add_action( 'wp_ajax_gc-message-box-deactivate', array($this,'deactivate') );

        }

        public function activate(){
            global $GC_Message_Box_Config;
            $helper = Gc_MessageBox_CF::create_and_init("Mygetconversion_Helper",array(
                "domain_url" => $this->get_domain_url(),
                "api_url" => $GC_Message_Box_Config['GCAPI'],
                "request" => Gc_MessageBox_CF::create("Request")
            ));
            $helper->handle_activate();
        }

        public function deactivate(){
            global $GC_Message_Box_Config;

            $helper = Gc_MessageBox_CF::create_and_init("Mygetconversion_Helper",array(
                "domain_url" => $this->get_domain_url(),
                "api_url" => $GC_Message_Box_Config['GCAPI'],
                "request" => Gc_MessageBox_CF::create("Request")
            ));
            $helper->handle_deactivate();
        }


        protected function initialize_less_js(){
            global $GC_Message_Box_Config;

            $less_helper = Gc_MessageBox_CF::create_and_init("Lessjs_Helper",array(
                "less_file" => plugins_url('gc-message-box/css/style-gc-message-box-admin'),
                "plugin_name" => "gc-message-box"
                ) );

            if($GC_Message_Box_Config['GCTYPE'] == "DEV"){
                $less_helper->set_debug_mode();
            }

            $less_helper->initialize();
            

        }

        public function info(){
            die('{"ver":"'.Gc_MessageBox_Util::get_version().'"}');  
		}

        public function remote(){
            $remote_handler = new Gc_Message_Box_Remote_Handler($this->namespace);
            $remote_handler->execute(Gc_MessageBox_CF::create("Request"));
            exit();
        }


        private function get_domain_url() {
            $pageURL = 'http';
            if ($_SERVER["HTTPS"] == "on") {$pageURL .= "s";}
                $pageURL .= "://";
            if ($_SERVER["SERVER_PORT"] != "80") {
                $pageURL .= $_SERVER["SERVER_NAME"].":".$_SERVER["SERVER_PORT"];
            } else {
                $pageURL .= $_SERVER["SERVER_NAME"];
            }
            return $pageURL;
        }

        public function group(){
            $request = Gc_MessageBox_CF::create("Request");
            if(!$request->has_param('id')){
                return;
            }
            if(!$request->has_param('group')){
                return;
            }
            $option = Gc_MessageBox_CF::create_and_init("Option",array("namespace" => $this->namespace,"id" => $request->get_param('id')."_group"));
            $option->set_value($request->get_param('group'));
            $option->save();
            return;
        }

        public function admin_init() {
            add_settings_section( 'style_settings', 'Style Settings', array($this, 'style_sub_message'), 'style_options');
            add_settings_section( 'general_settings', 'General Settings', array($this, 'general_sub_message'), 'general_options');
        }

        public function add_sub_menu() {
            add_submenu_page( 'plugins.php', 'GC Message Box Options', 'GC Message Box','manage_options', 'gc_message_box', array($this, 'sub_menu_page'));
        }

        public function sub_menu_page() {
            $this->init_values();
            if ( !current_user_can( 'manage_options' ) )  {
                wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
            }
            $renderer = new Gc_Message_Box_Admin_Renderer($this->namespace,$this,$this->options);
			$renderer->render("");
        }
		
		public function plugin_options_url() {
            return Gc_MessageBox_Util::plugin_options_url();
        }


        public function add_action_link($links, $file) {
            if ( $file == $this->this_plugin ) {
                $settings_link = '<a href="' . $this->plugin_options_url() . '">' . __( 'Settings' ) . '</a>';
                array_unshift( $links, $settings_link );
            }
            return $links;
        }

        public function init_values() {
            $installedOption = $this->options->get("installed");
            if ($installedOption->get_value() == "true") {
                return;
            }
            $options = $this->get_options();
            foreach ($options as $key => $opt) {
                $opt->save();
            }
            $installedOption->set_value("true");
            $installedOption->save();
        }

        public function get_options(){
            return $this->options->get_options();

        }    

        public function style_sub_message() {
            __( 'Options for styling the GC Message Box' );
        }
        
        public function general_sub_message() {
            __( 'General options for the GC Message Box' );
        }


		public function scripts_init(){
            $this->init_fonts();

            wp_enqueue_script( 'jquery' );

            wp_enqueue_style( 'wp-color-picker' );
            wp_enqueue_script( 'wp-color-picker' );


            wp_enqueue_script( 'gc_message_box_admin', plugins_url('gc-message-box/js/gc-admin.js') );
            wp_localize_script( 'gc_message_box_admin', 'WP', array(
                'base_url' => admin_url("admin-ajax.php"),
                'group_ajax_url' => 'gc-message-box-group',
                'disconnect_action' => 'gc-message-box'

            ) );
            Gc_Message_Box_Admin_Bar::script_init();

            $this->init_fonts_selector_scripts();
		}

        protected function init_fonts_selector_scripts(){
            wp_enqueue_script( 'jqueryselectbox',plugins_url('gc-message-box/js/jquery.selectbox.js' ));
            wp_enqueue_style( 'jqueryselectbox-css', plugins_url('gc-message-box/css/jquery.selectbox.css') );
            wp_register_style( 'google_webfonts', 'http://fonts.googleapis.com/css?family=Droid+Sans:400,700|Lato:100,300,400,700,900,100italic,300italic,400italic,700italic,900italic|PT+Sans:400,700,400italic,700italic|Bitter:400,700,400italic|Droid+Serif:400,700,700italic,400italic|Open+Sans:300italic,400italic,600italic,700italic,800italic,400,800,700,600,300|Oswald:400,700,300|Open+Sans+Condensed:300,300italic,700|Yanone+Kaffeesatz:400,700,300,200|Roboto:400,900italic,700italic,900,700,500italic,500,400italic,300italic,300,100italic,100&subset=latin,latin-ext,cyrillic,cyrillic-ext,greek-ext,greek,vietnamese' );
            wp_enqueue_style('google_webfonts');

        }

        protected function init_fonts(){
            wp_register_style( 'gc_message_box_font_roboto', 'http://fonts.googleapis.com/css?family=Roboto:400,700,300');
            wp_register_style( 'gc_message_box_font_roboto_condensed', 'http://fonts.googleapis.com/css?family=Roboto+Condensed:400,700,300');
            wp_enqueue_style( 'gc_message_box_font_roboto' );
            wp_enqueue_style( 'gc_message_box_font_roboto_condensed' );

        }

		public function handle_request(){
            //var_dump($_REQUEST);
            //var_dump($this->namespace.'submit');
            if (!isset($_REQUEST[$this->namespace.'submit']) or !isset($_REQUEST['post'])) {
                return;
            }

            $options = $this->get_options();
			
            foreach($options as $option) {
                if(!$option->is_visible()){
                    continue;
                }
                if (isset($_REQUEST['post'][$option->get_name()])) {
                    $option->set_value($_REQUEST['post'][$option->get_name()]);
                } elseif($option->is_checkbox()) {
                    $option->set_checked(false);
                }else{
                    continue;
                }
                $option->save();
            }
		}


        public function init_admin_bar() {
            $bar = new Gc_Message_Box_Admin_Bar();
            $bar->initialize($this->plugin_options_url());

        }
	}
}