<?php
if(!class_exists("Gc_Message_Box_Controller")){
    class Gc_Message_Box_Controller implements Gc_MessageBox_Controller_Interface {
        protected $namespace;
        protected $first_appear;
        protected $enable_repeat;
        protected $repeat_after;
        protected $gci = 0;
        protected $inloop = false;
        protected $renderer = null;
        protected $metrix_code = null;
        protected $metrix_enable = false;
        
        public function __construct($namespace){
            $this->namespace = $namespace;

        }

        public function initialize(){
            $options = Gc_MessageBox_CF::create("Option_Repository_Factory")->get_instance()->get_namespace($this->namespace);
            $this->first_appear = $options->get("first_appear");
            $this->enable_repeat = $options->get("enable_repeat");
            $this->repeat_after = $options->get("repeat_after");
            $this->metrix_code = $options->get("metrix_code");
            $this->metrix_enable = ($this->metrix_code->get_value() != "");
            $this->renderer = new Gc_Message_Box_Renderer($this->namespace, $this);

        }

        public function initialize_hooks(){
            add_action( 'init', array($this, 'scripts_init') );
            add_action( 'the_post', array($this,'put_after_the_post'), 1 );
            add_action( 'loop_start', array($this,'put_loop_start'), 1 );
            add_action( 'loop_end', array($this,'put_loop_end'), 1 );
			add_action( 'pre_get_posts', array($this,'reset_loop'), 1 );
            add_filter( 'comments_array', array($this,'comments_list_before'), 1 );
            add_action( 'wp_before_admin_bar_render', array($this, 'init_admin_bar') );

        }

        public function handle_request(){

        }
        
        public function plugin_options_url() {
            return Gc_MessageBox_Util::plugin_options_url();
        }

        public function scripts_init(){
            global $GC_Message_Box_Config;
            wp_register_style( 'gc_message_box_generated_style', plugins_url('gc-message-box/style-gc-message-box.php') );
            wp_enqueue_style( 'gc_message_box_generated_style' );

            Gc_Message_Box_Admin_Bar::script_init();

            if($this->metrix_enable){
                wp_enqueue_script( 'metrix', $GC_Message_Box_Config['METRIX_JS_URL']);
            }


        }
		
		public function reset_loop() {
			wp_reset_query();
		}

        public function put_loop_start($query) {
			global $wp_the_query;
			
            $this->gci = 0;
            if ( ($query === $wp_the_query) && $query->is_main_query()) {
                $this->inloop = true;
            }
        }


        public function put_after_the_post(){
            //Loop end
            if(!$this->inloop){
                return;
            }
            //Post page
            if(is_single()){
                return;
            }
            //Appear here test
            if (!$this->is_appear_here_enabled()) {
                return;
            }

            if($this->is_loop_counter_equal_first_appear()) {
               echo $this->renderer->render($this->gci);
            }
            if($this->is_repeat_enable()){
                if($this->is_loop_counter_less_first_appear()){
                    if($this->is_repeat_position()){
                        echo $this->renderer->render($this->gci);
                    }
                }
            }

            $this->gci++;

        }

        public function put_loop_end(  ) {
            $this->inloop = false;
        }

        public function comments_list_before($my_comments){
            
            if ($this->is_appear_here_enabled()) {
                echo $this->renderer->render($this->gci);
                echo $this->render_metrix_tracker();
            }
            return $my_comments;
        }

        public function render_metrix_tracker(){
            global $GC_Message_Box_Config;
            if(!$this->metrix_enable){
                return;
            }

            $metrix_helper = Gc_MessageBar_CF::create_and_init("Metrix_Helper",array(
                "endpoint_url" => $GC_Message_Box_Config['METRIX_ENDPOINT_URL'],
                "click_id" => "gc_message_box_button_a",
                "metrix_code" => $this->metrix_code
                )
            );
            $metrix_helper->render();

        }
        
        public function is_list() {
            return (is_home() || is_category() || is_search() || is_author() || is_archive() || is_tag());
        }
        
        public function is_article() {
            return is_single();
        }

        public function is_page() {
            return is_page();
        }


        protected function is_loop_counter_equal_first_appear(){
            return ($this->gci == $this->first_appear->get_value());
        }

        protected function is_loop_counter_less_first_appear(){
            return ((int)$this->first_appear->get_value() < $this->gci);
        }

        protected function is_repeat_enable(){
            return ($this->enable_repeat->is_checked());
        }

        protected function is_repeat_position(){
            return ((($this->gci - $this->first_appear->get_value()) % $this->repeat_after->get_value()) == 0);
        }

        protected function is_appear_here_enabled() {
            $options = Gc_MessageBox_CF::create("Option_Repository_Factory")->get_instance()->get_namespace($this->namespace);
            if ($options->get("appear_here")->is_checked()) {
                return true;
            }
            $page_functions = array(
                "is_home"       =>  "appear_here_home",
                "is_category"   =>  "appear_here_category",
                "is_tag"        =>  "appear_here_tag",
                "is_archive"    =>  "appear_here_archive",
                "is_author"     =>  "appear_here_author",
                "is_search"     =>  "appear_here_search",
                "is_single"     =>  "appear_here_article",
                "is_page"       =>  "appear_here_pages"
            );
            $appear_pages = $options->filter("get_group", "general_appear_here");
            foreach ($page_functions as $function => $opt) {
                if($function()) {
                    return ($appear_pages[$opt]->is_checked());
                }
            }   
            return false;
        }

        public function init_admin_bar() {
            $bar = new Gc_Message_Box_Admin_Bar();
            $bar->initialize($this->plugin_options_url());

        }

    }
}